##
# File:    ObjectExtractorTests.py
# Author:  J. Westbrook
# Date:    25-Apr-2019
#
# Updates:
#
##
"""
Tests for extractor selected values from collections (limited tests from mock-data repos)
"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"


import logging
import os

import time
import unittest

from rcsb.exdb.utils.ObjectExtractor import ObjectExtractor
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.config.ConfigUtil import ConfigUtil

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class ObjectExtractorTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(ObjectExtractorTests, self).__init__(methodName)
        self.__verbose = False

    def setUp(self):
        #
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        #
        configName = "site_info_remote_configuration"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        #
        self.__workPath = "."
        self.__mU = MarshalUtil(workPath=self.__workPath)
        self.__entityTaxonPath = os.path.join(self.__workPath, "entity_taxon.tdd")
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)\n", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testExtractEntityTaxonomyContent(self):
        """Test case - extract unique entity source and host taxonomies"""
        tL = []
        try:
            obEx = ObjectExtractor(
                self.__cfgOb,
                databaseName="pdbx_core",
                collectionName="pdbx_core_polymer_entity",
                useCache=False,
                keyAttribute="entity",
                uniqueAttributes=["rcsb_id"],
                selectionQuery=None,
                selectionList=["rcsb_id", "rcsb_entity_source_organism.ncbi_taxonomy_id", "rcsb_entity_host_organism.ncbi_taxonomy_id"],
            )
            eCount = obEx.getCount()
            logger.info("Polymer entity count is %d", eCount)
            objD = obEx.getObjects()
            sD = {}
            hD = {}
            for rId, eD in objD.items():
                try:
                    for tD in eD["rcsb_entity_source_organism"]:
                        sD.setdefault(rId, []).append(str(tD["ncbi_taxonomy_id"]))

                except Exception:
                    pass
                try:
                    for tD in eD["rcsb_entity_host_organism"]:
                        hD.setdefault(rId, []).append(str(tD["ncbi_taxonomy_id"]))
                except Exception:
                    pass
            for rId, taxIdL in sD.items():
                tS = "|".join(sorted(set(taxIdL)))
                if tS:
                    lS = "%s\t%s" % (rId, "|".join(sorted(set(taxIdL))))
                tL.append(lS)
            self.__mU.doExport(self.__entityTaxonPath, tL, fmt="list")
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def objectExtractorSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ObjectExtractorTests("testExtractEntityTaxonomyContent"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = objectExtractorSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File:    UpdateReferenceSequencesTests.py
# Author:  J. Westbrook
# Date:    12-Oct-2019
#
# Updates:
#
##
"""
Tests for reference sequence assignment update operations
"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os
import time
import unittest

from rcsb.exdb.seq.ReferenceSequenceAssignmentUpdater import ReferenceSequenceAssignmentUpdater
from rcsb.utils.config.ConfigUtil import ConfigUtil

logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class ReferenceSequenceAssignmentUpdaterTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(ReferenceSequenceAssignmentUpdaterTests, self).__init__(methodName)
        self.__verbose = True

    def setUp(self):
        #
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        configName = "site_info_configuration"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        #
        self.__resourceName = "MONGO_DB"
        self.__cachePath = os.path.join(TOPDIR, "CACHE")
        self.__testEntityCacheKwargs = {"fmt": "json", "indent": 3}
        self.__fetchLimitTest = None
        #
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)\n", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    def testUpdateAssignments(self):
        """Test case - get reference sequences and update candidates"""
        try:
            rsau = ReferenceSequenceAssignmentUpdater(self.__cfgOb, useCache=False, cachePath=self.__cachePath, fetchLimit=self.__fetchLimitTest, siftsAbbreviated="TEST")
            updateLimit = None
            updateId = "2019_01"
            lenUpd, numUpd = rsau.doUpdate(updateId, updateLimit=updateLimit)
            logger.info("Update length %d numUpd %d", lenUpd, numUpd)
            # self.assertEqual(numUpd, lenUpd)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def referenceUpdaterSuite():
    suiteSelect = unittest.TestSuite()
    suiteSelect.addTest(ReferenceSequenceAssignmentUpdaterTests("testUpdateAssignments"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = referenceUpdaterSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)



##
# File: TreeNodeListWorker.py
# Date: 9-Apr-2019  jdw
#
# Loading worker for tree node list data.
#
# Updates:
#  9-Sep-2019 jdw add AtcProvider() and ChemrefExtractor() for ATC tree.
# 12-Apr-2023 dwp add CARD ontology tree
#  8-Aug-2023 dwp Load full (unfiltered) taxonomy tree node list, and stop loading GO tree (will be loaded in DW instead)
# 27-Aug-2024 dwp Update CARD ontology tree loading
# 23-Jan-2025 dwp Change indexed field from 'update_id' to 'id'
#  7-Aug-2025 dwp Change target DB and collection names to "dw" and "tree_*" (via configuration file);
#                 Make use of configuration file for loading tree node lists and setting indexed fields
#  6-Jan-2026 dwp Raise error if a tree node list is empty or fails to load;
#                 Add support for providing a manual list of tree node lists to load (for testing)
#
##
__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"

import logging
import os.path

from rcsb.db.mongo.DocumentLoader import DocumentLoader
from rcsb.db.processors.DataExchangeStatus import DataExchangeStatus
from rcsb.exdb.chemref.ChemRefExtractor import ChemRefExtractor
from rcsb.utils.chemref.AtcProvider import AtcProvider
from rcsb.utils.ec.EnzymeDatabaseProvider import EnzymeDatabaseProvider
from rcsb.utils.targets.CARDTargetOntologyProvider import CARDTargetOntologyProvider
from rcsb.utils.struct.CathClassificationProvider import CathClassificationProvider
from rcsb.utils.struct.EcodClassificationProvider import EcodClassificationProvider
from rcsb.utils.struct.ScopClassificationProvider import ScopClassificationProvider
from rcsb.utils.struct.Scop2ClassificationProvider import Scop2ClassificationProvider
from rcsb.utils.taxonomy.TaxonomyProvider import TaxonomyProvider
from rcsb.exdb.seq.TaxonomyExtractor import TaxonomyExtractor

logger = logging.getLogger(__name__)


class TreeNodeListWorker(object):
    """Prepare and load repository holdings and repository update data."""

    def __init__(
            self,
            cfgOb,
            cachePath,
            numProc=1,
            chunkSize=10,
            maxStepLength=4000,
            readBackCheck=False,
            documentLimit=None,
            verbose=False,
            useCache=False,
            useFilteredLists=False,
            treeCollectionList=None,
    ):
        self.__cfgOb = cfgOb
        self.__cachePath = os.path.abspath(cachePath)
        self.__readBackCheck = readBackCheck
        self.__numProc = numProc
        self.__chunkSize = chunkSize
        self.__maxStepLength = maxStepLength
        self.__documentLimit = documentLimit
        self.__resourceName = "MONGO_DB"
        self.__filterType = "assign-dates"
        self.__verbose = verbose
        self.__statusList = []
        self.__useCache = useCache
        self.__useFilteredLists = useFilteredLists
        self.__treeCollectionList = treeCollectionList if treeCollectionList else []  # Manually specify tree collections to load

    def __updateStatus(self, updateId, databaseName, collectionName, status, startTimestamp):
        try:
            sFlag = "Y" if status else "N"
            desp = DataExchangeStatus()
            desp.setStartTime(tS=startTimestamp)
            desp.setObject(databaseName, collectionName)
            desp.setStatus(updateId=updateId, successFlag=sFlag)
            desp.setEndTime()
            self.__statusList.append(desp.getStatus())
            return True
        except Exception as e:
            logger.exception("Failing with %s", str(e))
        return False

    def load(self, updateId, loadType="full", doLoad=True):
        """Load tree node lists and status data -

        Relevant configuration options:

        tree_node_lists_configuration:
            DATABASE_NAME: dw
            COLLECTION_VERSION_STRING: 2.1.0
            COLLECTION_NAME_LIST:
                - tree_taxonomy
                - tree_ec
                - tree_scop
                - tree_scop2
                - tree_cath
                - tree_atc
                - tree_card
                - tree_ecod
            COLLECTION_INDICES:
                - INDEX_NAME: primary
                ATTRIBUTE_NAMES:
                    - id
                - INDEX_NAME: index_2
                ATTRIBUTE_NAMES:
                    - parents
        """
        ok = True
        useCache = self.__useCache
        #
        logger.info("Starting with cache path %r (useCache=%r)", self.__cachePath, useCache)
        #
        self.__statusList = []
        desp = DataExchangeStatus()
        statusStartTimestamp = desp.setStartTime()
        dl = DocumentLoader(
            self.__cfgOb,
            self.__cachePath,
            self.__resourceName,
            numProc=self.__numProc,
            chunkSize=self.__chunkSize,
            maxStepLength=self.__maxStepLength,
            documentLimit=self.__documentLimit,
            verbose=self.__verbose,
            readBackCheck=self.__readBackCheck,
        )
        #
        sectionName = "tree_node_lists_configuration"
        databaseNameMongo = self.__cfgOb.get("DATABASE_NAME", sectionName=sectionName)
        collectionNameList = self.__cfgOb.get("COLLECTION_NAME_LIST", sectionName=sectionName)
        collectionIndexList = self.__cfgOb.get("COLLECTION_INDICES", sectionName=sectionName)
        # databaseNameMongo = 'dw'
        # collectionNameList = ['tree_taxonomy', 'tree_ec', 'tree_scop', 'tree_scop2', 'tree_cath', 'tree_atc', 'tree_card', 'tree_ecod', 'tree_go']
        # collectionIndexList = [{'INDEX_NAME': 'primary', 'ATTRIBUTE_NAMES': ['id']}, {'INDEX_NAME': 'index_2', 'ATTRIBUTE_NAMES': ['parents']}]

        if len(self.__treeCollectionList) > 0:
            collectionNameList = [col for col in self.__treeCollectionList]

        # collectionVersion = self.__cfgOb.get("COLLECTION_VERSION_STRING", sectionName=sectionName)
        # addValues = {"_schema_version": collectionVersion}
        addValues = None

        for collectionName in collectionNameList:
            nL = self.__getTreeDocList(collectionName, useCache)
            if doLoad:
                if not nL or len(nL) == 0:
                    logger.error("Empty node list returned for collectionName %r", collectionName)
                    ok = False
                else:
                    okL = dl.load(
                        databaseNameMongo,
                        collectionName,
                        loadType=loadType,
                        documentList=nL,
                        keyNames=None,
                        addValues=addValues,
                        schemaLevel=None,
                        indexDL=collectionIndexList
                    )
                    ok = okL and ok
                    self.__updateStatus(updateId, databaseNameMongo, collectionName, ok, statusStartTimestamp)

                    logger.info(
                        "Completed load of tree node list for database %r, collection %r, len(nL) %r (status %r)",
                        databaseNameMongo, collectionName, len(nL), okL
                    )
        # ---
        logger.info("Completed tree node list loading operations with loadType %r (status %r)", loadType, ok)
        if not ok:
            raise ValueError("Failed to load at least one tree node list.")
        return ok

    def __checkTaxonNodeList(self, nL):
        eCount = 0
        tD = {dD["id"]: True for dD in nL}
        for dD in nL:
            if "parents" in dD:
                pId = dD["parents"][0]
                if pId not in tD:
                    logger.info("Missing parent for taxon %d", pId)
                    eCount += 1
            else:
                logger.info("No parents for node %r", dD["id"])

    def getLoadStatus(self):
        return self.__statusList

    def __getTreeDocList(self, collectionName, useCache):
        nL = []
        if collectionName.lower() == "tree_cath":
            ccu = CathClassificationProvider(cachePath=self.__cachePath, useCache=useCache)
            nL = ccu.getTreeNodeList()
        elif collectionName.lower() == "tree_scop2":
            scu2 = Scop2ClassificationProvider(cachePath=self.__cachePath, useCache=useCache)
            nL = scu2.getTreeNodeList()
        elif collectionName.lower() == "tree_scop":
            scu = ScopClassificationProvider(cachePath=self.__cachePath, useCache=useCache)
            nL = scu.getTreeNodeList()
        elif collectionName.lower() == "tree_ecod":
            ecu = EcodClassificationProvider(cachePath=self.__cachePath, useCache=useCache)
            nL = ecu.getTreeNodeList()
        elif collectionName.lower() == "tree_ec":
            edbu = EnzymeDatabaseProvider(cachePath=self.__cachePath, useCache=useCache)
            nL = edbu.getTreeNodeList()
        elif collectionName.lower() == "tree_card":
            okCou = True
            cou = CARDTargetOntologyProvider(cachePath=self.__cachePath, useCache=useCache)
            if not cou.testCache():
                ok = cou.buildOntologyData()
                cou.reload()
                if not (ok and cou.testCache()):
                    logger.error("Skipping load of CARD Target Ontology tree data because it is missing.")
                    okCou = False
            if okCou:
                nL = cou.getTreeNodeList()
        elif collectionName.lower() == "tree_taxonomy":
            tU = TaxonomyProvider(cachePath=self.__cachePath, useCache=useCache)
            if self.__useFilteredLists:
                # Get the taxon coverage in the current data set -
                epe = TaxonomyExtractor(self.__cfgOb)
                tL = epe.getUniqueTaxons()
                logger.info("Taxon coverage length %d", len(tL))
                #
                fD = {1}
                for taxId in tL:
                    fD.update({k: True for k in tU.getLineage(taxId)})
                logger.info("Taxon filter dictionary length %d", len(fD))
                logger.debug("fD %r", sorted(fD))
                #
                nL = tU.exportNodeList(filterD=fD)
            else:
                # Get the full taxon node list without filtering
                nL = tU.exportNodeList()
            self.__checkTaxonNodeList(nL)
        elif collectionName.lower() == "tree_atc":
            crEx = ChemRefExtractor(self.__cfgOb)
            atcFilterD = crEx.getChemCompAccessionMapping("ATC")
            logger.info("Length of ATC filter %d", len(atcFilterD))
            atcP = AtcProvider(cachePath=self.__cachePath, useCache=useCache)
            nL = atcP.getTreeNodeList(filterD=atcFilterD)
        else:
            logger.error("Unsupported tree node collection %r", collectionName)
        #
        logger.info("Gathered tree nodes for loading collection %s (length %d)", collectionName, len(nL))
        return nL



##
# File:    EntityInstanceExtractorTests.py
# Author:  J. Westbrook
# Date:    25-Mar-2019
#
# Updates:
# 21-Apr-2019 jdw Tests for full cache construction and processiong
#
##
"""
Tests for extractor of selected values from entity polymer collections (full cache)

"""

__docformat__ = "google en"
__author__ = "John Westbrook"
__email__ = "jwest@rcsb.rutgers.edu"
__license__ = "Apache 2.0"


import logging
import os
import time
import unittest


from rcsb.exdb.seq.EntityPolymerExtractor import EntityPolymerExtractor
from rcsb.utils.config.ConfigUtil import ConfigUtil
from rcsb.utils.io.MarshalUtil import MarshalUtil
from rcsb.utils.taxonomy.TaxonomyUtils import TaxonomyUtils


logging.basicConfig(level=logging.INFO, format="%(asctime)s [%(levelname)s]-%(module)s.%(funcName)s: %(message)s")
logger = logging.getLogger()

HERE = os.path.abspath(os.path.dirname(__file__))
TOPDIR = os.path.dirname(os.path.dirname(os.path.dirname(HERE)))


class EntityPolymerExtractorFullTests(unittest.TestCase):
    def __init__(self, methodName="runTest"):
        super(EntityPolymerExtractorFullTests, self).__init__(methodName)
        self.__verbose = True

    def setUp(self):
        #
        #
        self.__mockTopPath = os.path.join(TOPDIR, "rcsb", "mock-data")
        configPath = os.path.join(TOPDIR, "rcsb", "mock-data", "config", "dbload-setup-example.yml")
        #
        # Caution: this is very site specific setting
        #
        configName = "site_info_remote"
        self.__cfgOb = ConfigUtil(configPath=configPath, defaultSectionName=configName, mockTopPath=self.__mockTopPath)
        if configName != "site_info_configuration":
            self.__cfgOb.replaceSectionName("site_info_configuration", configName)
        #
        #
        self.__workPath = os.path.join(HERE, "test-cache-preserve")
        #
        self.__fullCacheKwargs = {"fmt": "pickle"}
        self.__fullEntitySaveCachePath = os.path.join(self.__workPath, "entity-polymer-data-cache.pic")
        #
        self.__mU = MarshalUtil()
        self.__entryLimitFull = 50
        #
        self.__startTime = time.time()
        logger.debug("Starting %s at %s", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()))

    def tearDown(self):
        endTime = time.time()
        logger.info("Completed %s at %s (%.4f seconds)\n", self.id(), time.strftime("%Y %m %d %H:%M:%S", time.localtime()), endTime - self.__startTime)

    @unittest.skip("rebuild cache")
    def testRebuildCache(self):
        """Test case - extract entity polymer info - rebuild full cache of extracted entity polymer data -"""
        try:
            epe = EntityPolymerExtractor(
                self.__cfgOb, saveCachePath=self.__fullEntitySaveCachePath, useCache=False, saveCacheKwargs=self.__fullCacheKwargs, entryLimit=self.__entryLimitFull
            )
            eCount = epe.getEntryCount()
            if self.__entryLimitFull is not None:
                self.assertGreaterEqual(eCount, self.__entryLimitFull)
            else:
                self.assertGreaterEqual(eCount, 10)
            #
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testAccessEntityPolymerFeatures(self):
        """Test case - access cached entity polymer info from full cache"""
        try:
            epe = EntityPolymerExtractor(self.__cfgOb, saveCachePath=self.__fullEntitySaveCachePath, useCache=True, saveCacheKwargs=self.__fullCacheKwargs)
            eCount = epe.getEntryCount()
            logger.info("Entry count %d", eCount)
            self.assertGreaterEqual(eCount, self.__entryLimitFull)
            #
            unpL = epe.getRefSeqAccessions("UNP")
            logger.info("Ref seq count %d", len(unpL))
            self.assertGreaterEqual(len(unpL), 1)
            #
            testOp = False
            if testOp:
                for entryId in ["1CP9"]:
                    for entityId in ["1", "2"]:
                        uL = epe.getEntityRefSeqAccessions("UNP", entryId, entityId)
                        logger.debug("UNP for %s %s %r", entryId, entityId, uL)
                #
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testAccessEntityPolymerReadCache(self):
        """Test case - access cached entity polymer info from full cache"""
        try:
            epe = EntityPolymerExtractor(self.__cfgOb, saveCachePath=self.__fullEntitySaveCachePath, useCache=True, saveCacheKwargs=self.__fullCacheKwargs)
            logger.info("Cache entry count %d", epe.getEntryCount())
            cD = epe.countRefSeqAccessions("UNP")
            self.assertGreaterEqual(len(cD), 2)
            #
            logger.info("UNP reference sequences per entity %r", dict(sorted(cD.items())))
            logger.info("Reference sequences per entity %r", dict(sorted(epe.countRefSeqAccessionAny().items())))
            logger.info("Reference sequences per ref db %r", dict(sorted(epe.countRefSeqAccessionDbType().items())))
            #
            ok = epe.checkRefSeqAlignRange("UNP")
            self.assertTrue(ok)
            unpL = epe.getRefSeqAccessions("UNP")
            logger.info("Unique UNP reference sequences %d", len(unpL))
            self.assertTrue(ok)
            #
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()

    def testTaxonomyEntityPolymerReadCache(self):
        """Test case - evaluate taxonomy - from full cache"""
        try:
            taxIdList = [562, 9606, 3701]
            for taxId in taxIdList:
                tU = TaxonomyUtils(taxDirPath=self.__workPath)
                tL = tU.getLineage(taxId)
                logger.info("Taxonomy lineage for %d %r", taxId, tL)
                #
                #
                epe = EntityPolymerExtractor(self.__cfgOb, saveCachePath=self.__fullEntitySaveCachePath, useCache=True, saveCacheKwargs=self.__fullCacheKwargs)
                logger.info("Cache entry count %d", epe.getEntryCount())
                logger.info("Reference sequences per ref db %r", dict(sorted(epe.countRefSeqAccessionDbType().items())))
                rD = epe.countRefSeqAccessionByTaxon(dbNameList=["UNP"])
                logger.info("Unique taxons %d", len(list(rD.keys())))
                #
                numT = 0
                for tId, aL in rD.items():
                    tL = tU.getLineage(tId)
                    if taxId in tL:
                        tc = len(set(aL))
                        logger.info("Matched %5d %s (%r)", tc, tU.getScientificName(tId), tId)
                        numT += tc
                logger.info("Total matched accessions %d ", numT)
        except Exception as e:
            logger.exception("Failing with %s", str(e))
            self.fail()


def entityPolymerExtractFullSuite():
    suiteSelect = unittest.TestSuite()
    # suiteSelect.addTest(EntityPolymerExtractorFullTests("testRebuildCache"))
    suiteSelect.addTest(EntityPolymerExtractorFullTests("testAccessEntityPolymerFeatures"))
    suiteSelect.addTest(EntityPolymerExtractorFullTests("testAccessEntityPolymerReadCache"))
    suiteSelect.addTest(EntityPolymerExtractorFullTests("testTaxonomyEntityPolymerReadCache"))
    return suiteSelect


if __name__ == "__main__":
    mySuite = entityPolymerExtractFullSuite()
    unittest.TextTestRunner(verbosity=2).run(mySuite)
